@@ -28,6 +28,7 @@ int write_jpeg_dct(
 );
 
 // ----------- RGB -------------
+typedef unsigned long BITMASK;
 int read_jpeg_spatial(
     const char *srcfile,
     unsigned char *rgb,
@@ -36,7 +37,7 @@ int read_jpeg_spatial(
     int out_color_space,
     int dither_mode,
     int dct_method,
-    unsigned long flags
+    BITMASK flags
 );
 
 int write_jpeg_spatial(
@@ -51,7 +52,7 @@ int write_jpeg_spatial(
     unsigned short *qt,
     short quality,
     short smoothing_factor,
-    unsigned long flags
+    BITMASK flags
 );
 
 //int jpeg_lib_version(void) { return JPEG_LIB_VERSION; }
