@@ -175,7 +175,7 @@ class JPEG:
         return spatial
 
     def write_spatial(self, dstfile, data=None, in_color_space=None, dct_method="JDCT_ISLOW",
-                      samp_factor=None, qt=100, smoothing_factor=None, flags=[]):
+                      samp_factor=None, qt=-1, smoothing_factor=None, flags=[]):
         """Writes spatial image representation (i.e. RGB) to a file.
         
         :param dstfile: Destination file name.
@@ -188,7 +188,7 @@ class JPEG:
         :type dct_method: str, optional
         :param samp_factor: Sampling factor. None, or list of 3 tuples of 2 ints. According to source by default. Read more at `glossary <https://jpeglib.readthedocs.io/en/latest/glossary.html#jpeg-sampling-factor>`_.
         :type samp_factor: list, optional
-        :param qt: Compression quality, between 0 and 100 or a tensor with quantization tables. Defaultly 100 (full quality).
+        :param qt: Compression quality, between 0 and 100 or a tensor with quantization tables. Defaultly -1 (default factor kept).
         :type qt: int | numpy.ndarray, optional
         :param smoothing_factor: Smoothing factor, between 0 and 100. Using default from libjpeg by default.
         :type smoothing_factor: int, optional
@@ -385,8 +385,8 @@ class JPEG:
         dct_method = self.J_DCT_METHOD[dct_method]
         in_cmap = None
         if colormap is not None:
-            if 'QUANTIZE_COLORS' not in flags:
-                flags.append('QUANTIZE_COLORS')
+            if 'QUANTIZE_COLORS' not in flags and '+QUANTIZE_COLORS' not in flags:
+                flags.append('+QUANTIZE_COLORS')
             in_cmap = np.ascontiguousarray(colormap)
             in_cmap = np.ctypeslib.as_ctypes(in_cmap.astype(np.ubyte))
         # allocate
@@ -409,7 +409,7 @@ class JPEG:
         )
         # align rgb
         data = np.ctypeslib.as_array(self._im_spatial).astype(np.ubyte)
-        if 'QUANTIZE_COLORS' in flags:
+        if 'QUANTIZE_COLORS' in flags and '+QUANTIZE_COLORS' in flags:
             data = data.reshape(self.channels,data.shape[2],-1)[0]
             # parse colormap
             colormap = np.ctypeslib.as_array(self._im_colormap)
